#include <avr/interrupt.h>
#include <inttypes.h>
//#include
#include <avr/io.h>
#include "DS18B20.h"
#include "lcd.h"
#include <stdio.h>

/*
Timer 0 prescaler=8-> 1Tick= 2,170s
F_CPU= 3,6864 MHz
Timeing in s:
A = 6~;
B = 64;
C = 60;
D = 10;
E = 9;
F = 55;
G = 0;
H = 480;
I = 70;
J = 410;
*/

#define DSPORT PORTE
#define DSPin 0
#define Ausgang DDRE|=(1<<DSPin)//Ausgang
#define Eingang DDRE&=~(1<<DSPin)//Eingang
#define set_high DSPORT|=(1<<DSPin)
#define set_low DSPORT&=~(1<<DSPin)
//warten mit counter
#define A 3
#define B 30
#define C 28
#define D 5
#define E 9
#define F 25
#define H 222
#define I 32
#define J 189
//warten mit _delay_s
/*#define A 6
#define B 64
#define C 60
#define D 10
#define E 9
#define F 55
#define G 0
#define H 480
#define I 70
#define J 410*/

int time_measurment=0;
int8_t messwertGanzz;
double messwert;

void temperatur ()
{	
	if ((time_measurment>138)||((time_measurment>11)&&(time_measurment<14)))
	{
		int LSB,MSB;
		init_measurment();
		if (!reset())
		{
			if (time_measurment>138)
			{
				writeByte (0xCC);//Skip Rom
				writeByte (0x44);//Convert T

			}
			if ((time_measurment>11)&&(time_measurment<14))
			{
				time_measurment=14;
				writeByte (0xCC);//Skip Rom
				writeByte (0xBE);//Read Scratchpad
				LSB=readByte();
				MSB=readByte();
				leave_measurement ();
				messwertGanzz=0xFF;
				messwertGanzz&=((MSB<<4)|(LSB>>4));
				messwert=((LSB & (1 << 3))*0.5+(LSB & (1 << 2))*0.25+(LSB & (1 << 1))*0.125+(LSB & (1 << 0))*0.0625);
				messwert+=messwertGanzz;
				lcd_gotoxy(0,0);
				char buffer [10];
				sprintf(buffer, "Ist: %-2f C",messwert);
				lcd_puts(buffer);
			}
		}
		leave_measurement ();
	}
}
void init_measurment ()
{
	//sicherstellen, dass kein ie strt
	//PCINT 0 deaktivieren
	GICR&=~(1<<PCIE0);
	//TC2 Overflow interrupt deaktivieren
	TIMSK&=~(1<<TOIE2);
	//TC2 OC interrupt aktivieren
	TIMSK|=(1<<OCIE2);
	//TC2, prescaler 8
	TCCR2&=~(1<<CS20);;
	TCCR2|=(1<<CS21);	
}
void leave_measurement ()
{
	//interrupts aktivieren
	GICR|=(1<<PCIE0);
	TIMSK|=(1<<TOIE2);
	//TC2 OC interrupt deaktivieren
	TIMSK&=~(1<<OCIE2);
	//TC2, prescaler 1024
	TCCR2|=(1<<CS20)|(1<<CS21)|(1<<CS22);
}
int reset ()
{
	int status;
	Ausgang;
	set_low;
	delay (H);
	Eingang;
	delay (I);
	status=PINE & 0x01;
	delay (J);
	return status;
}
void writeBit (int bit)
{
	if (bit) 
	{	//Schreibe '1'
		Ausgang;
		set_low;
		delay (A);
		Eingang;
		delay (B);
	}
	else
	{	//Schreibt '0'		
		Ausgang;
		set_low;
		delay (C);
		Eingang;
		delay (D);
	}
}
int readBit ()
{
	int bit;
	Ausgang;
	set_low;
	delay (A);
	Eingang;
	delay (E);
	bit=PINE & 0x01;
	delay (F);
	return bit;
}
void writeByte (byte)
{
	int loop;
	for (loop=0;loop<8;loop++)
	{
		writeBit (byte & 0x01);
		byte>>=1;
	}
}
int readByte()
{
	int loop,result=0;

	for (loop=0;loop<8;loop++)
	{
		result>>=1;
		if (readBit())
			result |=0x80;
	}
	return result;
}

void delay (int delay_time)			
{
	TCNT2=0;
	OCR2=delay_time;
	while (!(TIFR & (1 << OCF2)));
	TIFR|=(1<<OCF2);
}	
